#!/usr/bin/python
# -*- coding: utf-8 -*-

__copyright__ = """

    Copyright 2019 Samapriya Roy
    Copyright 2022 SatAgro, Krzysztof Stopa

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"

import os
import argparse
from datetime import datetime

from planet_basemap.mosaic.geom_rbox import idl
from planet_basemap.mosaic.download import download_aoi_file_mosaic_quads, download_list_mosaic_quads
from planet_basemap.mosaic.metadata import get_file_mosaic_quads_metadata


PL_API_KEY = os.getenv('PL_API_KEY', '')


def rbox_from_parser(args):
    idl(infile=args.geometry)


def mosaic_list_from_parser(args):
    start_date = datetime.strptime(args.start, '%Y-%m-%d').date()
    end_date = datetime.strptime(args.end, '%Y-%m-%d').date()
    quads = get_file_mosaic_quads_metadata(args.geometry, start_date, end_date, args.coverage,
                                          args.intersect_exact, args.api_key)
    quads.to_file(args.output)
    print(f"{quads.shape[0]} quads saved to file {args.output}")


def download_mosaic_from_parser(args):
    if args.list and os.path.exists(args.list):
        download_list_mosaic_quads(args.list, args.output, args.override)
    elif args.geometry and os.path.exists(args.geometry):
        if args.start and args.end:
            start_date = datetime.strptime(args.start, '%Y-%m-%d').date()
            end_date = datetime.strptime(args.end, '%Y-%m-%d').date()
            download_aoi_file_mosaic_quads(args.geometry, args.output, start_date, end_date, args.coverage,
                                           args.intersect_exact, args.override, args.api_key)
        else:
            print("Missing parameters!")
    else:
        print("Please specify either a geometry or a mosaic list valid filepath.")


def main():
    parser = argparse.ArgumentParser(description='Planet Mosaic Quads Download CLI')
    subparsers = parser.add_subparsers()
    # Quad bounding box checking parameters
    parser_rbox = subparsers.add_parser('rbox',
                                        help='Prints bounding box for geometry')
    parser_rbox.add_argument('--geometry',
                             help='Path to geometry file with the Area of Interest (AOI)'
                                  ' supports GeoJSON, KML, Shape (any supported by GDAL)')
    parser_rbox.set_defaults(func=rbox_from_parser)
    # Quad listing parameters
    parser_mosaic_list = subparsers.add_parser('list', help='Tool to get Mosaic Quads for given AOI')
    parser_mosaic_list.add_argument('--geometry', help='Path to AOI geometry file (any supported by GDAL)',
                                    required=True)
    parser_mosaic_list.add_argument('--start', help='Choose Start date in format YYYY-MM-DD',
                                    required=True)
    parser_mosaic_list.add_argument('--end', help='Choose End date in format YYYY-MM-DD',
                                    required=True)
    parser_mosaic_list.add_argument('--coverage', help="Choose minimum percentage coverage",
                                    default=0)
    parser_mosaic_list.add_argument('--intersect_exact', action='store_true', required=False,
                                    help='Filter quads that intersects with AOI. If not given quads for '
                                         'entire AOI bounding box are returned.')
    parser_mosaic_list.add_argument('--output', help='Full path where you want your mosaic list exported',
                                    required=True)
    parser_mosaic_list.add_argument('--api_key', help='Planet API key. Also can be set as PL_API_KEY env var.',
                                    default=PL_API_KEY)
    parser_mosaic_list.set_defaults(func=mosaic_list_from_parser)
    # Quad download parameters
    parser_download = subparsers.add_parser('download', help='Download quad GeoTiffs choose for given geometry or list')
    parser_download.add_argument('--geometry', help='Path to AOI geometry file (any supported by GDAL)',
                                 default=None, required=False)
    parser_download.add_argument('--list', help="Mosaic list where results from list command where saved",
                                 default=None, required=False)
    parser_download.add_argument('--start', help='Choose Start date in format YYYY-MM-DD', required=False)
    parser_download.add_argument('--end', help='Choose End date in format YYYY-MM-DD', required=False)
    parser_download.add_argument('--coverage', help="Choose minimum percentage coverage", default=0)
    parser_download.add_argument('--intersect_exact', action='store_true', required=False,
                                 help='Filter quads that intersects with AOI. If not given quads for '
                                      'entire AOI bounding box are returned.')
    parser_download.add_argument('--override', action='store_true', required=False,
                                 help='To override already downloaded quads.')
    parser_download.add_argument('--output', help='Local folder where downloaded data will be stored')
    parser_download.add_argument('--api_key', help='Planet API key. Also can be set as PL_API_KEY env var.',
                                    default=PL_API_KEY)
    parser_download.set_defaults(func=download_mosaic_from_parser)

    args = parser.parse_args()

    try:
        args.func(args)
    except AttributeError as e:
        print(f'You must provide a function. Try "list" or "download". Error {e}')


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

__copyright__ = """

    Copyright 2019 Samapriya Roy
    Copyright 2022 SatAgro, Krzysztof Stopa
    
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"

import os
from datetime import datetime

import pandas as pd
import geopandas as gpd
import requests
import pyproj
from functools import partial
from shapely.geometry import shape
from shapely.geometry import Polygon
from shapely.ops import transform
from shapely.geometry import box

PL_API_KEY = os.getenv('PL_API_KEY', '')


class DateRange:
    """
    Helper class for manage date ranges
    """

    def __init__(self, dt1, dt2):
        self._dt1 = dt1
        self._dt2 = dt2

    def __contains__(self, dt):
        return self._dt1 <= dt <= self._dt2


def _handle_page(response, geom_main_bound, start, end, min_coverage=0, api_key=PL_API_KEY):
    quads = gpd.GeoDataFrame()
    time_range = DateRange(start, end)
    for mosaic in response['mosaics']:
        bd = mosaic['bbox']
        mosaic_geom = shape(Polygon(box(bd[0], bd[1], bd[2], bd[3]).exterior.coords))
        gboundlist = geom_main_bound.split(',')
        boundgeom = shape(
            Polygon(box(float(gboundlist[0]), float(gboundlist[1]), float(gboundlist[2]), float(gboundlist[3]))))
        proj = partial(pyproj.transform, pyproj.Proj(init='epsg:4326'), pyproj.Proj(init='epsg:3857'))
        boundgeom = transform(proj, boundgeom)
        mosaic_geom = transform(proj, mosaic_geom)
        first_acquired = datetime.strptime(mosaic['first_acquired'].split('T')[0], '%Y-%m-%d').date()
        last_acquired = datetime.strptime(mosaic['last_acquired'].split('T')[0], '%Y-%m-%d').date()
        mosaic_in_range = first_acquired in time_range and last_acquired in time_range
        if boundgeom.intersection(mosaic_geom).is_empty:
            print('Error: empty bounding box!')
        elif mosaic_in_range:
            mosaic_id = mosaic['id']
            r = requests.get('https://api.planet.com/basemaps/v1/mosaics/' + str(mosaic_id) + '/quads?bbox=' + str(
                gboundlist[0]) + '%2C' + gboundlist[1] + '%2C' + gboundlist[2] + '%2C' + gboundlist[3],
                             auth=(api_key, ''))
            resp = r.json()
            for quad in resp['items']:
                if quad['percent_covered'] >= min_coverage:
                    quad_df = gpd.GeoDataFrame({
                        'mosaic_id': [str(mosaic['id'])],
                        'mosaic_name': [str(mosaic['name'])],
                        'quad_id': [quad['id']],
                        'geometry': box(quad['bbox'][0], quad['bbox'][1], quad['bbox'][2], quad['bbox'][3]),
                        'first_acquired': first_acquired.strftime('%Y-%m-%d'),
                        'last_acquired': last_acquired.strftime('%Y-%m-%d'),
                        'resolution': str(mosaic['grid']['resolution']),
                        'percent_covered': quad['percent_covered'],
                        'link_self': quad['_links']['_self'],
                        'link_thumbnail': quad['_links']['thumbnail'],
                        'link_item': quad['_links']['download'],
                        'link_download': quad['_links']['download'],
                    }, crs="EPSG:4326", index=[0, 2])
                    quads = pd.concat([quads, quad_df])
                    quads.reset_index()
    return quads


def get_file_mosaic_quads_metadata(filepath, start_date, end_date, min_coverage=0,
                                   intersect_exact=False, api_key=PL_API_KEY):
    """
    Get quads available for download at the AOIs defined in given vector file.
    :param filepath: File with defined AOIs polygons.
    :param start_date: Start date to filter quatds.
    :param end_date: End date.
    :param min_coverage: Minimum coverage of data.
    :param intersect_exact: To filter quads that not overlaps directly with AOIs if False bounding box is used.
    :param api_key: Planet API key.
    :return: GeoDataFrame with found quad data.
    :raises: PermissionError, FileNotFoundError, ValueError
    """
    quads = gpd.GeoDataFrame()
    if os.path.exists(filepath):
        aoi_data = gpd.read_file(filepath)
        for aoi in aoi_data.itertuples():
            aoi_mosaics = get_aoi_mosaic_quads_metadata(aoi.geometry, start_date, end_date, min_coverage, api_key)
            quads = pd.concat([quads, aoi_mosaics])
            quads.reset_index()
        if intersect_exact:
            quads = aoi_data.overlay(quads)
    else:
        raise FileNotFoundError(f"File {filepath} does not exists!")
    return quads


def get_aoi_mosaic_quads_metadata(aoi_geom, start, end, coverage=0, api_key=PL_API_KEY):
    """
    Get area of interest metadata.
    :param aoi_geom: Shapely shape
    :param start: Start date.
    :param end: End date.
    :param coverage: Minimum coverage percent of a quad.
    :param api_key: Planet API key.
    :return: Pandas DataFrame with found mosaic quads metadata.
    :raises: PermissionError, ValueError
    """
    quads = gpd.GeoDataFrame()
    gmainbound = (','.join(str(v) for v in list(aoi_geom.bounds)))
    print('rbox:' + str(gmainbound) + '\n')
    r = requests.get('https://api.planet.com/basemaps/v1/mosaics', auth=(api_key, ''))
    response = r.json()
    if 'message' in response:
        raise PermissionError('No Download permission for: Error: ' + str(response['message']))

    if 'mosaics' in response and response['mosaics'][0]['quad_download'] == True:
        resp_mosaics = _handle_page(response, gmainbound, start, end, coverage, api_key)
        quads = pd.concat([quads, resp_mosaics])
        quads.reset_index()
    else:
        raise ValueError(f"Missing mosaics data. Get: {response}")

    while response['_links'].get('_next') is not None:
        page_url = response['_links'].get('_next')
        r = requests.get(page_url)
        response = r.json()
        if response['mosaics'][0]['quad_download'] == True:
            resp_mosaics = _handle_page(response, gmainbound, start, end, coverage, api_key)
            quads = pd.concat([quads, resp_mosaics])
            quads.reset_index()
    return quads.drop_duplicates()



import os
import shutil
import unittest

from datetime import date

from planet_basemap.mosaic.download import download_aoi_file_mosaic_quads
from planet_basemap.mosaic.metadata import get_file_mosaic_quads_metadata

CLEANUP = False


class TestMosaicMetadata(unittest.TestCase):

    def _test_environment(self):
        self.assertIsNotNone(os.getenv('PL_API_KEY'),
                             "Set Planet API key environment variable PL_API_KEY to run tests.")

    def test_aoi_mosaic_metadata(self):
        self._test_environment()
        start_date = date(2022, 1, 1)
        end_date = date(2022, 3, 1)
        quads = get_file_mosaic_quads_metadata('./data/test_aoi_01.geojson', start_date, end_date)
        self.assertGreaterEqual(quads.shape[0], 50, "Missing mosaic metadata")
        mosaics_intersect = get_file_mosaic_quads_metadata('./data/test_aoi_01.geojson',
                                                           start_date, end_date, intersect_exact=True)
        self.assertGreater(quads.shape[0], mosaics_intersect.shape[0], "Intersecting quads should be less than bbox.")

    def test_aoi_download_shape(self):
        """
        Test to get metadata and download geometry with multiple polygons in a shape file
        """
        self._test_environment()
        start_date = date(2022, 1, 1)
        end_date = date(2022, 3, 1)
        # TODO

    def test_aoi_download(self):
        self._test_environment()
        test_aoi_file = './data/test_aoi_02.geojson'
        start_date = date(2022, 1, 1)
        end_date = date(2022, 3, 1)
        output = './data/downloaded'
        if not os.path.exists(output):
            os.makedirs(output)
        down_files = download_aoi_file_mosaic_quads(test_aoi_file, output, start_date, end_date, intersect_exact=False,
                                                    override=False)
        print(f"Downloaded {len(down_files)} quads.")
        # Check files have been downloaded
        for img in down_files:
            self.assertTrue(os.path.exists(img))
        self.assertGreaterEqual(len(output), 12, "Missing downloaded files")
        # Clean up
        if CLEANUP:
            shutil.rmtree(output)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
import requests
import os
import geopandas as gpd

from planet_basemap.mosaic.metadata import get_file_mosaic_quads_metadata

PL_API_KEY = os.getenv('PL_API_KEY', '')


def download_mosaic_quads(quads_dataset, output, override=False):
    down_quad_paths = []
    for quad in quads_dataset.itertuples():
        print(f"Downloading {quad.link_download}")
        quad_path = download_quad(quad.mosaic_name, quad.link_download, output, override)
        if quad_path:
            down_quad_paths.append(quad_path)
    return down_quad_paths


def download_list_mosaic_quads(list_path, output, override=False):
    quads = gpd.read_file(list_path)
    return download_mosaic_quads(quads, output, override),


def download_aoi_file_mosaic_quads(aoi_path, output, start_date, end_date, coverage=0, intersect_exact=False,
                                   override=False, api_key=PL_API_KEY):
    """
    Download base map mosaics quads for given coverage in a vector file.
    :param aoi_path: Path to a vector file with coverage polygons (in CRS as EPSG:4326)
    :param output: Path where data will be downloaded.
    :param start_date: Start date.
    :param end_date: End date.
    :param coverage: Minimum coverage percentage.
    :param intersect_exact: True to get only intersecting quads otherwise polygon bounding box will be downloaded.
    :param override: True if you want to download again seme data.
    :param api_key: Planet API key.
    :return: List with downloaded quad file paths.
    """
    quads = get_file_mosaic_quads_metadata(aoi_path, start_date, end_date, coverage, intersect_exact, api_key)
    return download_mosaic_quads(quads, output, override)


def download_quad(name, download_link, output, override=False):
    r = requests.get(download_link, allow_redirects=False)
    file_link = r.headers['Location']
    file_name = str(file_link).split('%22')[-2]
    file_path = os.path.join(output, name)
    if not os.path.exists(file_path):
        os.makedirs(file_path)
    quad_path = os.path.join(file_path, file_name)
    if os.path.exists(quad_path) and not override:
        print("File already exists SKIPPING: " + str(quad_path))
        return quad_path
    else:
        result = requests.get(download_link, auth=(PL_API_KEY, ''))
        if result.status_code == 200:
            print("Saving to " + str(quad_path))
            f = open(quad_path, 'wb')
            for chunk in result.iter_content(chunk_size=512 * 1024):
                if chunk:
                    f.write(chunk)
            f.close()
            return quad_path
        else:
            print("Encountered error with code: " + str(result.status_code) + ' for ' + str(quad_path))
    return None
